import logging


logger = logging.getLogger('pacu')


def initialize_logging() -> None:
    """
    Initializes the logging.
    :return: None
    """
    formatter = logging.Formatter('%(asctime)s - %(module)15s - %(levelname)7s - %(message)s')

    # Console handler
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    console_handler.setLevel(logging.DEBUG)
    console_handler.name = 'console'
    logger.addHandler(console_handler)

    # File handler (pacu.log file)
    file_handler = logging.FileHandler('pacu.log')
    file_handler.setFormatter(formatter)
    file_handler.setLevel(logging.DEBUG)
    logger.addHandler(file_handler)

    # General logging level
    logger.setLevel(logging.DEBUG)
    logger.propagate = False



import logging
from typing import Optional, List

import shutil

from pathlib import Path

from pacu.app.report.htmlbase import HtmlBase
from pacu.app.report.htmlelement import HtmlElement


class HtmlReport(HtmlBase):
    """
    This class represents an HTML report.
    """

    def __init__(self, filename: Path, output_dir: Path = None, include_js: Optional[List[Path]] = None):
        """
        Initializes the report.
        :param filename: Filename
        :param output_dir: Output directory
        :param include_js: (Optional) List of Javascript files that are included in the report
        """
        super(HtmlReport, self).__init__()
        self._filename = filename
        self._output_dir = output_dir
        self._include_js = include_js

    @property
    def output_dir(self) -> Path:
        """
        Returns the output directory.
        :return: Output directory
        """
        return self._output_dir

    def save(self) -> None:
        """
        Saves the report.
        :return: None
        """
        logging.info(f"Saving report '{self._filename.name}'")
        if self._filename is None:
            raise ValueError("Report with filename 'None' cannot be saved")
        with self._filename.open('w', encoding='utf-8') as handle:
            self.add_raw('<!DOCTYPE HTML>')
            handle.write(self.to_html())
        if self._include_js:
            if self._output_dir is None:
                raise ValueError("Cannot enable Javascript when there is no output directory")
            for path in self._include_js:
                shutil.copyfile(path, self._output_dir / path.name)

    def initialize(self, title: str, css_style: Path = None) -> None:
        """
        Initializes an HTML report.
        :param title: Report title
        :param css_style: CSS style
        :return: None
        """
        logging.info("Initializing report")
        with self._doc.tag('head'):
            with self._doc.tag('title'):
                self._doc.text(title)
            self._doc.stag('meta', charset='UTF-8')
            if self._include_js:
                for path in self._include_js:
                    with self.get_tag('script', [('type', 'text/javascript'), ('src', path.name)]):
                        self._doc.text('')
            if css_style is not None:
                with css_style.open() as css, self._doc.tag('style', type="text/css"):
                    self._doc.asis(css.read())

    def add_pipeline_header(self, pipeline_name: str) -> None:
        """
        Adds the pipeline header to the report.
        :param pipeline_name: Name of the pipeline
        :return: None
        """
        if self._output_dir is None:
            raise ValueError("Can't add the pipeline header without an output directory")
        with self.get_tag('div', [('class', 'header')]):
            with self.get_tag('div', [('id', 'header_title')]):
                self.add_text(f"{pipeline_name} report")

    def to_html(self) -> str:
        """
        Returns the report HTML code.
        :return: HTML code.
        """
        parent = HtmlElement('html')
        parent.add_raw(self._doc.getvalue())
        return parent.to_html()



import tempfile
import unittest
from importlib.resources import files
from pathlib import Path

from pacu.app.utils import megautils


class TestMEGAUtils(unittest.TestCase):
    """
    Tests for the MEGA utils.
    """

    path_fasta = Path(str(files('pacu').joinpath('resources/testdata/snp_matrix.fasta')))

    def test_model_selection(self) -> None:
        """
        Tests the model selection functionality with regular input.
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_out = Path(dir_, 'model_selection.csv')
            megautils.run_model_selection(
                path_fasta=TestMEGAUtils.path_fasta,
                path_out=path_out,
                dir_=Path(dir_),
                branch_swap_filter='Very weak',
                missing_data_treatment='Partial deletion',
                site_cov_cutoff=50,
                threads=4
            )

            # Parse the output file
            model_info = megautils.parse_model_selection_csv(path_out)
            self.assertIn('model', model_info)
            self.assertIn('rates_among_sites', model_info)

    def test_tree_building(self) -> None:
        """
        Tests the tree building functionality with regular input.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_model_selection_csv = Path(str(files('pacu').joinpath(
                'resources/testdata/mega/model_selection_output.csv')))

            # Run tree building
            path_out_nwk = Path(dir_, 'tree.nwk')
            megautils.run_tree_building(
                path_fasta=TestMEGAUtils.path_fasta,
                path_csv=path_model_selection_csv,
                path_out=path_out_nwk,
                dir_=Path(dir_),
                branch_swap_filter='Weak',
                missing_data_treatment='Use all sites',
                site_cov_cutoff=None,
                bootstrap_replicates=100,
                heuristic_method='SPR3',
                initial_tree='NJ',
                gamma_categories=5,
                threads=4
            )

            # Verify that the output file is OK
            self.assertTrue(path_out_nwk.exists())
            self.assertGreater(path_out_nwk.stat().st_size, 0)


if __name__ == '__main__':
    unittest.main()



import re
from pathlib import Path
from typing import Union

from pacu import Command, logger


def run_ml_tree_construction(path_fasta: Path, path_out: Path, threads: int = 4) -> Command:
    """
    Runs the ML tree reconstruction using IQ-TREE.
    :param path_fasta: Input SNP matrix FASTA file
    :param path_out: Output path
    :param threads: Number of threads to use
    :return: Command used for tree construction
    """
    if not path_out.name.endswith('.nwk'):
        raise ValueError("Output path should end with the '.nwk' extension")
    basename = path_out.name.replace('.nwk', '')
    prefix = path_out.parent / basename
    command = Command(f"iqtree2 -s {path_fasta} --boot 100 -m MFP --prefix {prefix} -T {threads}")
    command.run(path_out.parent)
    if not command.exit_code == 0:
        raise RuntimeError(f'Error running IQ-TREE: {command.stderr}')
    Path(path_out.parent, f'{basename}.treefile').rename(path_out)
    return command


def extract_selected_model(stdout: str) -> Union[str, None]:
    """
    Extracts the information on the selected model for tree construction.
    :param stdout: Stdout
    :return: Name of the selected model
    """
    for line in stdout.splitlines():
        m = re.match('^Best-fit model: (.*?) chosen', line.strip())
        if not m:
            continue
        return m.group(1)
    logger.warning(f'Cannot extract selected model from IQ-TREE output')



import tempfile
import unittest
from importlib.resources import files
from pathlib import Path

from pacu import initialize_logging
from pacu.app.utils import bamutils


class TestBAMUtils(unittest.TestCase):
    """
    Tests the BAM utils module.
    """

    def test_add_custom_tag(self) -> None:
        """
        Tests add_custom_tag function.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_bam_in = Path(str(files('pacu').joinpath('resources/testdata/bam/ilmn/TIAC1151.bam')))
            path_bam_out = Path(dir_, 'updated.bam')
            bamutils.add_custom_tag('PACU_name', 'custom_sample', path_bam_in, path_bam_out)
            self.assertTrue(path_bam_out.exists())
            self.assertEqual(bamutils.read_custom_tag(path_bam_out, 'PACU_name'), 'custom_sample')


if __name__ == '__main__':
    initialize_logging()
    unittest.main()



import os
from pathlib import Path
from typing import Dict

from pacu import logger


def trimmomatic_dir_adapters() -> Path:
    """
    Retrieves the directory containing the trimmomatic adapters.
    :return: Directory containing trimmomatic adapters
    """
    # CONDA installation
    if os.environ.get('CONDA_PREFIX') is not None:
        return Path(os.environ['CONDA_PREFIX'], 'share', 'trimmomatic', 'adapters')
    try:
        return Path(os.environ['$TRIMMOMATIC_ADAPTER_DIR'])
    except KeyError:
        logger.error(f'TRIMMOMATIC_ADAPTER_DIR environment variable not set')
        raise RuntimeError(f'TRIMMOMATIC_ADAPTER_DIR environment variable not set')


def trimmomatic_collect_output(dir_out: Path) -> Dict[str, Path]:
    """
    Collects the trimmomatic output.
    :param dir_out: Output directory
    :return: Output dictionary
    """
    try:
        fq_dict = {
            '1P': next(dir_out.glob('*_1P.fastq.gz')),
            '2P': next(dir_out.glob('*_2P.fastq.gz'))
        }
    except StopIteration:
        raise FileNotFoundError(f'No paired end reads survived trimming')

    # Collect output (single-end reads)
    for key in ('1U', '2U'):
        try:
            path_fq = next(dir_out.glob(f'*_{key}.fastq.gz'))
            fq_dict[key] = path_fq
        except StopIteration:
            logger.warning(f"Trimmed unpaired reads ('{key}') not found")
            fq_dict[key] = None
    return fq_dict



import json
from importlib.resources import files
from typing import Dict, Any

from pacu.app.report.htmlbase import HtmlBase


class HtmlCitation(HtmlBase):
    """
    This class is used to format citations in HTML.
    :param citation_data: Citation data (parsed from RIS)
    :return: None
    """

    def __init__(self, citation_data: Dict[str, Any]) -> None:
        """
        Initializes the citation.
        :param citation_data: Citation data
        """
        super().__init__()
        self._citation_data = citation_data
        self._authors = '; '.join(citation_data['first_authors'])
        self._pub_year = citation_data['publication_year'].split('/')[0]
        self._title = citation_data['primary_title']
        citation_processor = {
            'jour': self._process_journal_citation,
            'book': self._process_book_citation}
        try:
            citation_processor[citation_data['type_of_reference'].lower()]()
        except KeyError as err:
            raise KeyError(
                f"Citations of type {citation_data['type_of_reference']} are not supported! (KeyError '{err}')")

    def _process_journal_citation(self) -> None:
        """
        Creates the html tags when the citation is for a journal article.
        :return: None
        """
        with self.get_tag('div', attributes=[('class', 'citations')]):
            with self.get_tag('p'):
                # Volume + (number optional)
                journal_parts = [
                    self._citation_data['alternate_title3'], f", {self._citation_data.get('volume', 'n/a')}"]
                if 'number' in self._citation_data:
                    journal_parts.append(f" ({self._citation_data['number']})")
                journal = ''.join(journal_parts)

                # Citation text
                self.add_text(f"{self._authors} ({self._pub_year}). {self._title}. In <i>{journal}</i>. ")

                # Citation DOI / link
                with self.get_tag('a', [('href', f"https://dx.doi.org/{self._citation_data['doi']}")]):
                    self.add_text(f"DOI: {self._citation_data['doi']}")

    def _process_book_citation(self) -> None:
        """
        Creates the html tags when the citation is for a book.
        :return: None
        """
        with self.get_tag('div', attributes=[('class', 'citations')]):
            with self.get_tag('p'):
                edition = f", {self._citation_data['edition']}" if 'edition' in self._citation_data else ''
                publisher = self._citation_data['publisher']
                self.add_text(f"{self._authors} ({self._pub_year}). {self._title}{edition}. {publisher}")

    @staticmethod
    def parse_from_json(json_basename: str) -> 'HtmlCitation':
        """
        Parses a HTML citation from a JSON file.
        :param json_basename: Basename for the JSON file
        :return: Citation
        """
        json_citation = files('pacu').joinpath(f'resources/citations/{json_basename}.json')
        with json_citation.open(encoding='utf-8') as handle:
            data = json.load(handle)
        return HtmlCitation(data)



import tempfile
import unittest
from importlib.resources import files
from pathlib import Path

from pacu import initialize_logging
from pacu.map_to_ref import MapToRef


class TestMapToRef(unittest.TestCase):
    """
    Tests the map to ref script.
    """

    def test_map_to_ref_illumina(self) -> None:
        """
        Tests the map to ref script with Illumina data.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_out = Path(dir_, 'mapped_reads.bam')
            map_to_ref = MapToRef([
                '--ref-fasta', str(files('pacu').joinpath('resources/testdata/NC_002695.2-subset.fasta')),
                '--read-type', 'illumina',
                '--fastq-illumina',
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151_1P.fastq.gz')),
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151_2P.fastq.gz')),
                '--output', str(path_out),
                '--dir-working', dir_,
                '--threads', '4'
            ])
            map_to_ref.run()

            # Verify the output file
            self.assertTrue(path_out.exists())
            self.assertGreater(path_out.stat().st_size, 0)

    def test_map_to_ref_with_fasta_name(self) -> None:
        """
        Tests the map to ref script with the fasta name parameter set.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_out = Path(dir_, 'mapped_reads.bam')
            map_to_ref = MapToRef([
                '--ref-fasta', str(files('pacu').joinpath('resources/testdata/NC_002695.2-subset.fasta')),
                '--ref-fasta-name', 'my_reference.fasta',
                '--read-type', 'illumina',
                '--fastq-illumina',
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151_1P.fastq.gz')),
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151_2P.fastq.gz')),
                '--output', str(path_out),
                '--dir-working', dir_,
                '--threads', '4'
            ])
            map_to_ref.run()

            # Verify the output file
            self.assertTrue(path_out.exists())
            self.assertGreater(path_out.stat().st_size, 0)

    def test_map_to_ref_illumina_with_trim(self) -> None:
        """
        Tests the map to ref script with Illumina data and trimming enabled.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_out = Path(dir_, 'mapped_reads.bam')
            map_to_ref = MapToRef([
                '--ref-fasta', str(files('pacu').joinpath('resources/testdata/NC_002695.2-subset.fasta')),
                '--read-type', 'illumina',
                '--fastq-illumina',
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151_1P.fastq.gz')),
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151_2P.fastq.gz')),
                '--output', str(path_out),
                '--trim',
                '--dir-working', dir_,
                '--threads', '4'
            ])
            map_to_ref.run()

            # Verify the output file
            self.assertTrue(path_out.exists())
            self.assertGreater(path_out.stat().st_size, 0)

    def test_map_to_ref_ont(self) -> None:
        """
        Tests the map to ref script with ONT data.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_out = Path(dir_, 'mapped_reads.bam')
            map_to_ref = MapToRef([
                '--ref-fasta', str(files('pacu').joinpath('resources/testdata/NC_002695.2-subset.fasta')),
                '--read-type', 'ont',
                '--fastq-ont',
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151-ont.fastq.gz')),
                '--output', str(path_out),
                '--dir-working', dir_,
                '--threads', '4'
            ])
            map_to_ref.run()

            # Verify the output file
            self.assertTrue(path_out.exists())
            self.assertGreater(path_out.stat().st_size, 0)

    def test_map_to_ref_ont_with_trim(self) -> None:
        """
        Tests the map to ref script with ONT data and trimming enabled.
        :return: None
        """
        with tempfile.TemporaryDirectory(prefix='pacu') as dir_:
            path_out = Path(dir_, 'mapped_reads.bam')
            map_to_ref = MapToRef([
                '--ref-fasta', str(files('pacu').joinpath('resources/testdata/NC_002695.2-subset.fasta')),
                '--read-type', 'ont',
                '--fastq-ont',
                str(files('pacu').joinpath('resources/testdata/fastq/TIAC1151-ont.fastq.gz')),
                '--trim',
                '--output', str(path_out),
                '--dir-working', dir_,
                '--threads', '4'
            ])
            map_to_ref.run()

            # Verify the output file
            self.assertTrue(path_out.exists())
            self.assertGreater(path_out.stat().st_size, 0)


if __name__ == '__main__':
    initialize_logging()
    unittest.main()



import re
import tempfile
from pathlib import Path

from pacu import Command


def add_custom_tag(name: str, value: str, bam_in: Path, bam_out: Path) -> None:
    """
    Adds the custom tag to the input BAM file.
    :param name: Tag name
    :param value: Tag value
    :param bam_in: Input BAM file
    :param bam_out: Output BAM file
    :return: None
    """
    with tempfile.TemporaryDirectory(prefix='pacu') as dir_temp:
        # Extract the original header
        command = Command(f'samtools view -H {bam_in}')
        command.run(Path(dir_temp))
        if not command.exit_code == 0:
            raise RuntimeError(f'Error extracting header from BAM file: {bam_in}')
        header = command.stdout

        # Add custom tag and save header
        header += f'@CO\t{name}:{value}\n'
        path_header_updated = Path(dir_temp, f'header_updated.txt')
        with path_header_updated.open('w') as handle:
            handle.write(header)

        # Create output BAM file with custom tag
        command = Command(f'samtools reheader {path_header_updated} {bam_in.absolute()} > {bam_out.absolute()}')
        command.run(Path(dir_temp))
        if not command.exit_code == 0:
            raise RuntimeError(f'Error replacing BAM header: {command.stderr}')


def read_custom_tag(bam_in: Path, name: str) -> str:
    """
    Reads a custom tag from the input BAM file.
    :param bam_in: Input BAM file
    :param name: Tag name
    :return: Tag value
    """
    command = Command(f'samtools view -H {bam_in}')
    command.run(bam_in.parent)
    if not command.exit_code == 0:
        raise RuntimeError(f'Cannot extract header from: {bam_in}')
    for line in command.stdout.splitlines():
        if not line.startswith('@CO'):
            continue
        m = re.match(r'@CO\t(.*):(.*)', line)
        if (m is None) or (m.group(1) != name):
            continue
        return m.group(2)
    raise ValueError(f"Cannot extract tag '{name}'")



from collections import defaultdict
from dataclasses import dataclass
from pathlib import Path
from typing import List, Dict, Optional

import pandas as pd
import vcf
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
# noinspection PyProtectedMember
from vcf.model import _Record as VCFRecord

from pacu.app.utils.loggingutils import logger


@dataclass(unsafe_hash=True, frozen=True, order=True)
class SNPPosition:
    contig: str
    position: int
    reference_base: chr


def parse_vcf_file(path_vcf: Path, include_filtered: bool) -> List[VCFRecord]:
    """
    Parses a single VCF file.
    :param path_vcf: VCF path
    :param include_filtered: If True, filtered variants are included
    :return: List of SNP positions
    """
    vcf_records = []
    logger.debug(f'Parsing VCF file: {path_vcf}')
    for record in list(vcf.Reader(filename=str(path_vcf))):
        # Remove non-SNP positions
        if not record.is_snp:
            continue
        # Remove filtered records (unless specified otherwise)
        if (not include_filtered) and len(record.FILTER) > 0:
            continue
        vcf_records.append(record)
    logger.info(f"{path_vcf.name} parsed: {len(vcf_records):,} variant positions")
    return vcf_records


def __get_nucleotides_per_position(paths_vcf: List[Path], names: List[str], include_filtered: bool = False) -> \
        Dict[SNPPosition, Dict[str, str]]:
    """
    Returns a dictionary with the nucleotide for each sample at each variant position.
    :param paths_vcf: List of input VCF files
    :param include_filtered: If True, filtered positions are retained
    :return: Sample_names, nucleotide per position per sample
    """
    nucl_by_position = defaultdict(dict)
    for path_vcf, name in zip(paths_vcf, names):
        for record in parse_vcf_file(path_vcf, include_filtered):
            position = SNPPosition(record.CHROM, record.POS, record.REF)
            if len(record.FILTER) > 0:
                nucl_by_position[position][name] = 'N'
            else:
                nucl_by_position[position][name] = str(record.ALT[0])

    # Remove positions with only N
    nucl_by_position = {pos: nucl for pos, nucl in sorted(nucl_by_position.items()) if not all(
        [x == 'N' for x in nucl.values()])}

    logger.info(f"{len(nucl_by_position):,} SNP positions found across all samples")
    return nucl_by_position


def create_snp_matrix(paths_vcf: List[Path], names: List[str], path_out: Path, path_tsv: Optional[Path] = None,
                      include_ref: bool = False) -> None:
    """
    Creates an SNP matrix from the input VCF file.
    :param paths_vcf: List of input VCF files
    :param names: Isolate names
    :param path_out: Output path
    :param path_tsv: TSV file with SNP positions
    :param include_ref: If True, the reference genome is included
    :return: None
    """
    nucl_by_pos = __get_nucleotides_per_position(paths_vcf, names, True)

    # Export TSV file with SNP positions
    if path_tsv is not None:
        data_positions = pd.DataFrame([{'seq_id': pos.contig, 'position': pos.position} for pos in nucl_by_pos.keys()])
        data_positions.to_csv(path_tsv, sep='\t', index=False)
        logger.info(f'SNP positions exported to: {path_tsv}')

    # Create FASTA records
    seq_by_sample_name = {name: [] for name in names}
    if include_ref is True:
        seq_by_sample_name['reference'] = []
    for snp_pos, nucl_by_sample in sorted(nucl_by_pos.items(), key=lambda x: x[0].position):
        for name in seq_by_sample_name.keys():
            seq_by_sample_name[name].append(nucl_by_sample.get(name, snp_pos.reference_base))

    # Write output file
    seqs = [SeqRecord(Seq(''.join(seq)), name, description='') for name, seq in seq_by_sample_name.items()]
    with path_out.open('w') as handle:
        SeqIO.write(seqs, handle, 'fasta')



from pathlib import Path

from setuptools import setup, find_packages

from pacu.version import __version__

with open(Path(__file__).parent / 'README.md', encoding='utf-8') as handle:
    long_description = handle.read()


setup(
    name='pacu_snp',
    version=__version__,
    description='Workflow for whole genome sequencing based phylogeny of Illumina and ONT data.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/BioinformaticsPlatformWIV-ISP/PACU/',
    author='Bert Bogaerts',
    author_email='bioit@sciensano.be',
    license='GPL',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
    ],
    keywords='nanopore illumina sequencing phylogeny',
    packages=find_packages() + ['pacu.resources'],
    python_requires='>=3',
    install_requires=[
        'PuLP==2.7.0',
        'PyYAML>=6.0',
        'UpSetPlot>=0.8.0',
        'beautifulsoup4>=4.11.1',
        'biopython',
        'matplotlib>=3.7.1',
        'numpy>=1.26.4',
        'pandas>=2.1.0',
        'pytest>=8.2.2',
        'pyvcf3>=1.0.3',
        'snakemake==7.18.2',
        'yattag>=1.14.0'
    ],
    package_data={
        'pacu': [
            'resources/citations/*.json',
            'resources/figtree_template.txt',
            'resources/mega/*.mao',
            'resources/snp_workflow.smk',
            'resources/style.css',
            'resources/testdata/bam/ilmn/*.bai',
            'resources/testdata/bam/ilmn/*.bam',
            'resources/testdata/bam/ont/*.bai',
            'resources/testdata/bam/ont/*.bam',
            'resources/testdata/fastq/*.fastq.gz',
            'resources/testdata/mega/model_selection_output.csv',
            'resources/testdata/vcfs/*.vcf',
            'resources/testdata/NC_002695.2-subset.fasta',
            'resources/testdata/phylogeny.nwk',
            'resources/testdata/snp_matrix.fasta',
        ]},
    entry_points={
        'console_scripts': [
            'PACU=pacu.run_pacu:main',
            'PACU_map=pacu.map_to_ref:main',
            'PACU_galaxy=pacu.galaxy:main',
        ],
    }
)



from pacu.app.report.htmlelement import HtmlElement


class HtmlTableCell(HtmlElement):
    """
    This class represents a cell in an HTML table.
    """

    def __init__(self, text, color=None, attributes=None, link=None):
        """
        Initializes a table cell.
        :param text: Text
        :param color: Color
        :param attributes: Attributes
        :param link: Link to add to the cell text
        """
        self._as_text = text
        if color is not None:
            color_attribute = [('class', color)]
            if attributes is not None:
                attributes += color_attribute
            else:
                attributes = color_attribute
        if link is not None:
            super().__init__('td', None, attributes)
            self.add_html_object(HtmlElement('a', text, [('href', link)]))
        else:
            super().__init__('td', text, attributes)

    @property
    def text(self):
        """
        Returns the text belonging to this tag.
        :return: Text
        """
        return self._as_text
